<?php
/**
 * Display a front-office block to list the latest post of a Wordpress blog
 *
 * See LICENCE.txt for terms of use
 * History :
 * 	@version 1.1 (2011-08-26) : 
 *		compatible Prestashop 1.3.x & 1.4.x
 *		simplify module structure
 *
 * TODO :
 *
 */
 
class pss_wpinps extends Module
{
    private $_postErrors = array();
	private $_moduleNames;
	// config default set
	private $_config = array(
		'PRESTASCOPE_WPIPS_NB_POSTS' => 5,					// number of posts to display in block
		'PRESTASCOPE_WPIPS_SUM_LEN' => 100,					// length of excerpt
		'PRESTASCOPE_WPIPS_WP_DIR' => '/blog/',				// WP install dir - relative to ps install dir

		'PRESTASCOPE_WPIPS_VERSION' => '1.1',
	);
	private $_versionHistory = array('0.1', '1.0', '1.1');
	private $_languages;
	private $_defaultFormLanguage;
	private $_allowUrlRewriting;
	// absolute URL to this module
	private $_shopUrl;
	private $_isPs14x;
	// the admin URL to get configuration screen for current module
	private $confUrl;
	
    function __construct()
    {
		global $cookie;
		
		$this->name = 'pss_wpinps';
		// some changes between 1.3.x (and previous) and 1.4.x Prestashop versions
		if ($this->isPs12x() || $this->isPs13x()) 
		{
			$this->tab = 'Prestascope';
		} 
		else if ($this->isPs14x()) 
		{
			$this->tab = 'front_office_features';
			$this->author = 'PrestaScope';
		}
        $this->version = $this->_config['PRESTASCOPE_WPIPS_VERSION'];
		
		// url rewriting active ?
		$this->_allowUrlRewriting = (int)Configuration::get('PS_REWRITING_SETTINGS');
		// full url & path to current module
		// Use Tools::getHttpHost(false, true).__PS_BASE_URI__
		$this->_shopUrl = $this->is_https()?'https':'http'.'://'.$_SERVER['HTTP_HOST'];
		$this->_isPs14x = $this->isPs14x();
		
        parent::__construct();

        $this->displayName = $this->l('PSS/ Block WordPress in PrestaShop');
//        $this->description = $this->l('This module displays a block listing the last posts of a Wordpress blog.');
		$buf = '<style type="text/css">';
		$buf .= 'a.descriptionLink {color:blue;text-decoration:underline;} a.descriptionLink:hover{color:red;text-decoration:none;}';
		$buf .= 'div.pssDescriptionDiv {background:#6a6a6a; color:white; padding:3px; margin-top:2px;}';
		$buf .= '</style>';
		$buf .= '<div class="pssDescriptionDiv">';
		$buf .= $this->l('This module displays a block listing the last posts of a');
		$buf .= ' <strong>';
		$buf .= $this->l('Wordpress blog');
		$buf .= '</strong>. ';
		$buf .= $this->l('Installation guide available on');
		$buf .= ' <a href="http://www.prestascope.com/blog/front-office-hacks/embed-a-wordpress-blog-into-a-prestashop-shop-2x/" target="_blank" class="descriptionLink">';
		$buf .= $this->l('Prestascope blog');
		$buf .= '</a>.';
		$buf .= '</div>';
        $this->description = $buf;

		// the admin URL to get configuration for current module
		$tab = Tools::getValue('tab');
		$token = Tools::getValue('token');
		$mainParts = explode('?', Tools::htmlentitiesUTF8($_SERVER['REQUEST_URI']));
		$this->confUrl = $mainParts[0].'?tab=AdminModules&configure='.$this->name.'&token='.$token;
    }
	private function is_https()
	{
		return strtolower(substr($_SERVER["SERVER_PROTOCOL"],0,5))=='https'? true : false;
	}

    function install()
    {
		// check required version - 1.2.x or 1.3.x or 1.4.x
		if (!$this->isPs12x() && !$this->isPs13x() && !$this->isPs14x())
			return false;
		// normal install
		if(	!parent::install() || 
			!$this->registerHook('home') ||
			!$this->_installConfig())		// to be done at last because deal with file versions in conf
			return false;
		return true;
    }
	/**
	 * Install the config constants for this module
	 */ 
	private function _installConfig() 
	{
		foreach ($this->_config as $key => $value) 
			Configuration::updateValue($key, $value);
		return true;
	}
	public function uninstall() 
	{
		if(	!parent::uninstall() || 
			!$this->_uninstallConfig())
			return false;			
		return true;
	}
	/**
	 * Uninstall the config constants for this module
	 */ 
	private function _uninstallConfig() 
	{
		foreach ($this->_config as $key => $value) 
			Configuration::deleteByName($key);
		return true;
	}
	
	// ******************************************************************************************
	//
	//
	// Display / manage configuration
	//
	//
	// ******************************************************************************************
	/**
	 * Manage display and processes
	 */
	public function getContent()
	{
		// languages management
		$allowEmployeeFormLang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
		if ($allowEmployeeFormLang && !$cookie->employee_form_lang)
			$cookie->employee_form_lang = intval(Configuration::get('PS_LANG_DEFAULT'));
		$useLangFromCookie = false;
		$this->_languages = Language::getLanguages();
		if ($allowEmployeeFormLang)
			foreach ($this->_languages AS $lang)
				if ($cookie->employee_form_lang == $lang['id_lang'])
					$useLangFromCookie = true;
		if (!$useLangFromCookie)
			$this->_defaultFormLanguage = intval(Configuration::get('PS_LANG_DEFAULT'));
		else
			$this->_defaultFormLanguage = intval($cookie->employee_form_lang);
		$this->_html .= '
		<script type="text/javascript">
			$(document).ready(function() {
				id_language = '.$this->_defaultFormLanguage.';
			});
		</script>';
		
		// configuration page title
		$this->_html .= '<h2>'.$this->l('Prestascope / Wordpress in Prestashop').'</h2><br /><br />';
		
		// deal with form submit
		if (Tools::isSubmit('submitGeneralSettings') || Tools::isSubmit('submitWpSettings'))
		{
			if (Tools::isSubmit('submitGeneralSettings'))
				$this->_postValidationGeneralSettings();
			else if (Tools::isSubmit('submitWpSettings'))
				$this->_postValidationWpSettings();
			// display errors if any
			if (!sizeof($this->_postErrors))
				$this->_postProcess();
			else
				foreach ($this->_postErrors AS $err)
					$this->_html .= '<div class="alert error">'.$err.'</div>';
		}

		$this->_displayForm();

		return $this->_html;
	}
	/**
	 * Validate user choices - FORM general settings
	 */
	private function _postValidationGeneralSettings()
	{
		if (!Validate::isInt(Tools::getValue('PRESTASCOPE_WPIPS_NB_POSTS')))
			$this->_postErrors[] = $this->l('The number of posts should be a valid integer');
		if (!Validate::isInt(Tools::getValue('PRESTASCOPE_WPIPS_SUM_LEN')))
			$this->_postErrors[] = $this->l('The length of post excerpts should be a valid integer');
	}
	/**
	 * Validate user choices - Wordpress settings
	 */
	private function _postValidationWpSettings()
	{
		$installDir = Tools::getValue('PRESTASCOPE_WPIPS_WP_DIR');
		if (!$installDir || strlen($installDir)<=0)
			$this->_postErrors[] = $this->l('The Wordpress installation directory is mandatory');
	}
	private function _postProcess()
	{
		if (Tools::isSubmit('submitGeneralSettings'))
		{
			Configuration::updateValue('PRESTASCOPE_WPIPS_NB_POSTS', (int)Tools::getValue('PRESTASCOPE_WPIPS_NB_POSTS'));
			Configuration::updateValue('PRESTASCOPE_WPIPS_SUM_LEN', (int)Tools::getValue('PRESTASCOPE_WPIPS_SUM_LEN'));

			$res = true;
			$this->_html .= '<h3 class="'. ($res ? 'conf confirm' : 'alert error') .'" style="margin-bottom: 20px">';
			$this->_html .= $res ? $this->l('General settings saved') : $this->l('Error while saving general settings');
			$this->_html .= '</h3>';
		}
		else if (Tools::isSubmit('submitWpSettings'))
		{
			// prepare the WP installation directory
			// add heading / if missing
			$pss_wpinps_install_dir = Tools::getValue('PRESTASCOPE_WPIPS_WP_DIR');
			if (substr($pss_wpinps_install_dir,0,1)!='/')
				$pss_wpinps_install_dir = '/'.$pss_wpinps_install_dir;
			// add trailing / if missing
			if (substr($pss_wpinps_install_dir,strlen($pss_wpinps_install_dir)-1,1)!='/')
				$pss_wpinps_install_dir = $pss_wpinps_install_dir.'/';
			Configuration::updateValue('PRESTASCOPE_WPIPS_WP_DIR', $pss_wpinps_install_dir);

			$res = true;
			$this->_html .= '<h3 class="'. ($res ? 'conf confirm' : 'alert error') .'" style="margin-bottom: 20px">';
			$this->_html .= $res ? $this->l('Wordpress settings saved') : $this->l('Error while saving Wordpress settings');
			$this->_html .= '</h3>';
		}
	}
	/**
	 * Display the configuration form.
	 */
	private function _displayForm()
	{
		global $link, $cookie;
		
		$iso = Language::getIsoById(intval($cookie->id_lang));		
		
		// get values from configuration
		$confs = Configuration::getMultiple(array('PRESTASCOPE_WPIPS_NB_POSTS','PRESTASCOPE_WPIPS_SUM_LEN','PRESTASCOPE_WPIPS_WP_DIR', 'PRESTASCOPE_WPIPS_DB_NAME','PRESTASCOPE_WPIPS_DB_USER','PRESTASCOPE_WPIPS_DB_PWD','PRESTASCOPE_WPIPS_DB_HOST'));
	
		// ------------------------------------------------------------
		// the slide show general settings
		$this->_html .= '
		<form method="post" action="'.$this->confUrl.'">
			<fieldset>
				<legend><img src="'.$this->_path.'wpmini-blue.png" />'.$this->l('General settings').'</legend>
				<label>'.$this->l('Number of posts').': </label>
				<div class="margin-form">
					<input type="text" size="4" maxlength="2" name="PRESTASCOPE_WPIPS_NB_POSTS" value="'.$confs['PRESTASCOPE_WPIPS_NB_POSTS'].'" /> <sup>*</sup>
					<p>'.$this->l('To set the maximum number of posts to display in the block').'</p>
				</div>
				<label>'.$this->l('Post excerpt length').': </label>
				<div class="margin-form">
					<input type="text" size="4" maxlength="3" name="PRESTASCOPE_WPIPS_SUM_LEN" value="'.$confs['PRESTASCOPE_WPIPS_SUM_LEN'].'" /> <sup>*</sup>
					<p>'.$this->l('The length of excerpt to display below each post title in the block. Set to 0 not to display the excerpt.').'</p>
				</div>
				<div class="margin-form clear pspace"><input type="submit" name="submitGeneralSettings" value="'.$this->l('Update').'" class="button" /></div>
			</fieldset>
		</form>';
		
		$this->_html .= '
		<div style="height:10px;">&nbsp;</div>
		<form method="post" action="'.$this->confUrl.'">
			<fieldset>
				<legend><img src="'.$this->_path.'wpmini-grey.png" />'.$this->l('Wordpress settings').'</legend>
				<label>'.$this->l('Installation dir').': </label>
				<div class="margin-form">
					<input type="text" size="100" maxlength="255" name="PRESTASCOPE_WPIPS_WP_DIR" value="'.$confs['PRESTASCOPE_WPIPS_WP_DIR'].'" /> <sup>*</sup>
					<p>'.$this->l('The Wordpress installation directory relative to the Prestashop installation directory').'</p>
				</div>
				<div class="clear"></div>
				<div class="margin-form clear pspace"><input type="submit" name="submitWpSettings" value="'.$this->l('Update').'" class="button" /></div>
			</fieldset>
		</form>';

		$this->_html .= '
		<div style="height:10px;">&nbsp;</div>
		<fieldset style="background:#FBF7EB;">
			<legend><img src="../img/admin/help-new.png" width="16" height="16" />'.$this->l('User Help').'</legend>
			<label>'.$this->l('Block position').' :</label>
			<div class="margin-form">
				<p class="pss_help">'.$this->l('The front-office block is activated by default in the right column. use the admin / modules / position tool to change. Only left or right column are supported.').'</p>
				<div class="clear"></div>
			</div>
			<label>'.$this->l('Block translations').' :</label>
			<div class="margin-form">
				<p class="pss_help">'.$this->l('The block translations might be changed in the "admin/tools/translations" screen, select "module" before clicking the flag. Then find the pss_wpinps group, enter the new translations and save.').'</p>
				<p class="pss_help">'.$this->l('To see the changes, don\'t forget to delete in your smarty/compile folder the cache file called xxx.file.blockpss_wpinps.tpl.php.').'</p>
				<div class="clear"></div>
			</div>
		</fieldset>';

		// --------------------------------------------
		// <Prestascope news iframe>
		// $this->_html .= '
		// <div style="height:10px;">&nbsp;</div>
		// <fieldset style="background:#FBF7EB;">
		// 	<legend><img style="vertical-align:middle;" src="'.$this->_path.'pss_new_32x3.png" width="32" height="32" />'.$this->l('Prestascope news').'</legend>
		// 	<iframe width="100%" height="200" style="border:none;" src="http://www.prestascope.com/pss_cms_content.php?cms=9&iso=en">
		// 	</iframe>
		// </fieldset>';
		// </Prestascope news iframe>

	}

	// ******************************************************************************************
	//
	//
	// Block display
	//
	//
	// ******************************************************************************************
	function hookHome($params)
	{
		$pss_wpinps_install_dir = '/blog/';
		if ($pss_wpinps_install_dir && strlen($pss_wpinps_install_dir)>0)
		{
		    if (file_exists(_PS_ROOT_DIR_.$pss_wpinps_install_dir.'wp-load.php') && !defined('PS_ADMIN_DIR') && !defined('_PS_ADMIN_DIR_')) // not in the admin section or ther will be a redeclaration error for a translate() function defined both in PS and WP
		    {
		        // load WordPress
		        define('WP_USE_THEMES', false);
		        require_once(_PS_ROOT_DIR_.$pss_wpinps_install_dir.'wp-load.php');
		    }
		}
		
		global $smarty, $cookie, $link;

		// extract criterias from configuration
		//
		$confs = Configuration::getMultiple(array('PRESTASCOPE_WPIPS_NB_POSTS','PRESTASCOPE_WPIPS_SUM_LEN','PRESTASCOPE_WPIPS_DB_NAME','PRESTASCOPE_WPIPS_DB_USER','PRESTASCOPE_WPIPS_DB_PWD','PRESTASCOPE_WPIPS_DB_HOST'));
		
		// display excerpt ?
		if (intVal($confs['PRESTASCOPE_WPIPS_SUM_LEN'])==0)
			$display_excerpt = FALSE;
		else
			$display_excerpt = TRUE;
		
		// WordPress API is already included in the config.inc.php file
		$blogLink = '.';
		$posts = array();
		if (function_exists ('esc_url') && function_exists ('home_url') && function_exists('get_posts') && function_exists('get_permalink'))
		{
			// the blog home page URL
			$blogLink = esc_url( home_url( '/' ) );

			// load latest posts from WP system
			$wp_posts = get_posts(array('numberposts' => intVal($confs['PRESTASCOPE_WPIPS_NB_POSTS'])));

			for($i=0;$i<count($wp_posts);$i++)
			{
				$posts[$i] = array();
				$posts[$i]['title'] = $wp_posts[$i]->post_title;
				$posts[$i]['link'] = get_permalink($wp_posts[$i]->ID);
				$postOutput = preg_replace('/<img[^>]+./', '', $wp_posts[$i]->post_content);
				$posts[$i]['excerpt'] = pss_wpinps::truncate($postOutput, intVal($confs['PRESTASCOPE_WPIPS_SUM_LEN']), '...');
			}
		}
		$blogLink = '/blog';
		// publish for template
		$smarty->assign(array(
			'blogLink'=>$blogLink,
			'posts' => $posts,
			'display_excerpt' => $display_excerpt
			));
				
		return $this->display(__FILE__, 'blockpss_wpinps.tpl');
	}
	


	// ******************************************************************************************
	//
	//
	// Tools for installation
	//
	//
	// ******************************************************************************************
	/**
	 * Check if installed Prestashop is a 1.2.x version
	 */
	public static function isPs12x() 
	{
		return self::checkPsVersion('1.2');
	}
	/**
	 * Check if installed Prestashop is a 1.3.x version
	 */
	public static function isPs13x() 
	{
		return self::checkPsVersion('1.3');
	}
	/**
	 * Check if installed Prestashop is a 1.4.x version
	 */
	public static function isPs14x() 
	{
		return self::checkPsVersion('1.4');
	}
	/**
	 * Check if installed Prestashop match an input radix
	 */
	public static function checkPsVersion($radixVersion) 
	{
		// get PS version
		$psVersion = _PS_VERSION_;
		if ($psVersion==null)
			return false;
		
		// look for version like 1.3.1, 1.3.7.0 or 1.4.3
		$subVersions = explode('.', $psVersion);
		$searchVersions = explode('.', $radixVersion);
		
		for ($i=0;$i<count($searchVersions);$i++) 
		{
			// compare each sub part of version
			if ($subVersions[$i] !== $searchVersions[$i]) 
				return false;
		}
		return true;
	}
	// ******************************************************************************************
	//
	//
	// Tools for installation
	//
	//
	// ******************************************************************************************
	/**
	 * Truncate a text or a HTML content closing HTML tags
	 * Thanks to Jonas Raoni Soares Silva
	 * See post at : http://snippets.dzone.com/posts/show/7125
	 */
	public static function truncate($text, $length, $suffix = '&hellip;', $isHTML = true)
	{
		$i = 0;
		$simpleTags=array('br'=>true,'hr'=>true,'input'=>true,'image'=>true,'link'=>true,'meta'=>true);
		$tags = array();
		if($isHTML)
		{
			preg_match_all('/<[^>]+>([^<]*)/', $text, $m, PREG_OFFSET_CAPTURE | PREG_SET_ORDER);
			foreach($m as $o)
			{
				if($o[0][1] - $i >= $length)
					break;
				$t = substr(strtok($o[0][0], " \t\n\r\0\x0B>"), 1);
				// test if the tag is unpaired, then we mustn't save them
				if($t[0] != '/' && (!isset($simpleTags[$t])))
					$tags[] = $t;
				elseif(end($tags) == substr($t, 1))
					array_pop($tags);
				$i += $o[1][1] - $o[0][1];
			}
		}
		
		// output without closing tags
		$output = substr($text, 0, $length = min(strlen($text),  $length + $i));
		// closing tags
		$output2 = (count($tags = array_reverse($tags)) ? '</' . implode('></', $tags) . '>' : '');
		
		// Find last space or HTML tag (solving problem with last space in HTML tag eg. <span class="new">)
		$pos = (int)end(end(preg_split('/<.*>| /', $output, -1, PREG_SPLIT_OFFSET_CAPTURE)));
		// Append closing tags to output
		$output.=$output2;

		// Get everything until last space
		$one = substr($output, 0, $pos);
		// Get the rest
		$two = substr($output, $pos, (strlen($output) - $pos));
		// Extract all tags from the last bit
		preg_match_all('/<(.*?)>/s', $two, $tags);
		// Add suffix if needed
		if (strlen($text) > $length) { $one .= $suffix; }
		// Re-attach tags
		$output = $one . implode($tags[0]);

		//added to remove  unnecessary closure
		$output = str_replace('</!-->','',$output); 

		return $output;
	}
}
?>
